@@ -27,9 +27,7 @@
 #include <locale.h>
 #include <canberra-gtk.h>
 
-#include "egg-debug.h"
-
-#include "gcm-cell-renderer-profile.h"
+#include "gcm-cell-renderer-profile-text.h"
 #include "gcm-calibrate-argyll.h"
 #include "gcm-cie-widget.h"
 #include "gcm-image.h"
@@ -37,7 +35,10 @@
 #include "gcm-profile-store.h"
 #include "gcm-trc-widget.h"
 #include "gcm-utils.h"
-#include "gcm-xyz.h"
+#include "gcm-color.h"
+#include "gcm-debug.h"
+
+static guint xid = 0;
 
 typedef struct {
 	GtkBuilder	*builder;
@@ -78,8 +79,6 @@ enum {
 	GCM_VIEWER_COMBO_COLUMN_LAST
 };
 
-static void gcm_viewer_profile_store_changed_cb (GcmProfileStore *profile_store, GcmViewerPrivate *viewer);
-
 #define GCM_VIEWER_TREEVIEW_WIDTH		350 /* px */
 #define GCM_VIEWER_MAX_EXAMPLE_IMAGES		4
 
@@ -106,7 +105,7 @@ gcm_viewer_error_dialog (GcmViewerPrivate *viewer, const gchar *title, const gch
 static void
 gcm_viewer_close_cb (GtkWidget *widget, GcmViewerPrivate *viewer)
 {
-	gtk_application_quit (viewer->application);
+	g_application_release (G_APPLICATION (viewer->application));
 }
 
 /**
@@ -157,7 +156,7 @@ gcm_viewer_preferences_cb (GtkWidget *widget, GcmViewerPrivate *viewer)
 	GError *error = NULL;
 	ret = g_spawn_command_line_async ("gnome-control-center color", &error);
 	if (!ret) {
-		egg_warning ("failed to run prefs: %s", error->message);
+		g_warning ("failed to run prefs: %s", error->message);
 		g_error_free (error);
 	}
 }
@@ -231,7 +230,7 @@ gcm_viewer_update_profile_list (GcmViewerPrivate *viewer)
 	gchar *sort = NULL;
 	GPtrArray *profile_array = NULL;
 
-	egg_debug ("updating profile list");
+	g_debug ("updating profile list");
 
 	/* get new list */
 	profile_array = gcm_profile_store_get_array (viewer->profile_store);
@@ -251,7 +250,7 @@ gcm_viewer_update_profile_list (GcmViewerPrivate *viewer)
 					gcm_viewer_profile_get_sort_string (profile_kind),
 					description);
 		filename = gcm_profile_get_filename (profile);
-		egg_debug ("add %s to profiles list", filename);
+		g_debug ("add %s to profiles list", filename);
 		gtk_list_store_set (viewer->list_store_profiles, &iter,
 				    GCM_PROFILES_COLUMN_ID, filename,
 				    GCM_PROFILES_COLUMN_SORT, sort,
@@ -274,8 +273,9 @@ gcm_viewer_profile_delete_cb (GtkWidget *widget, GcmViewerPrivate *viewer)
 	GtkWidget *dialog;
 	GtkResponseType response;
 	GtkWindow *window;
-	gint retval;
-	const gchar *filename;
+	GError *error = NULL;
+	GFile *file = NULL;
+	gboolean ret;
 	GcmProfile *profile;
 	GtkTreeSelection *selection;
 	GtkTreeModel *model;
@@ -301,7 +301,7 @@ gcm_viewer_profile_delete_cb (GtkWidget *widget, GcmViewerPrivate *viewer)
 	widget = GTK_WIDGET (gtk_builder_get_object (viewer->builder, "treeview_profiles"));
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget));
 	if (!gtk_tree_selection_get_selected (selection, &model, &iter)) {
-		egg_debug ("no row selected");
+		g_debug ("no row selected");
 		goto out;
 	}
 
@@ -311,10 +311,13 @@ gcm_viewer_profile_delete_cb (GtkWidget *widget, GcmViewerPrivate *viewer)
 			    -1);
 
 	/* try to remove file */
-	filename = gcm_profile_get_filename (profile);
-	retval = g_unlink (filename);
-	if (retval != 0)
+	file = gcm_profile_get_file (profile);
+	ret = g_file_delete (file, NULL, &error);
+	if (!ret) {
+		g_warning ("failed to be deleted: %s", error->message);
+		g_error_free (error);
 		goto out;
+	}
 out:
 	return;
 }
@@ -388,7 +391,7 @@ gcm_viewer_profile_import_file (GcmViewerPrivate *viewer, GFile *file)
 	/* check if correct type */
 	ret = gcm_utils_is_icc_profile (file);
 	if (!ret) {
-		egg_debug ("not a ICC profile");
+		g_debug ("not a ICC profile");
 		goto out;
 	}
 
@@ -418,7 +421,7 @@ gcm_viewer_profile_import_cb (GtkWidget *widget, GcmViewerPrivate *viewer)
 	/* get new file */
 	file = gcm_viewer_file_chooser_get_icc_profile (viewer);
 	if (file == NULL) {
-		egg_warning ("failed to get filename");
+		g_warning ("failed to get filename");
 		goto out;
 	}
 
@@ -448,7 +451,7 @@ gcm_viewer_drag_data_received_cb (GtkWidget *widget, GdkDragContext *context, gi
 		goto out;
 
 	/* import this */
-	egg_debug ("dropped: %p (%s)", data, filename);
+	g_debug ("dropped: %p (%s)", data, filename);
 
 	/* split, as multiple drag targets are accepted */
 	filenames = g_strsplit_set ((const gchar *)filename, "\r\n", -1);
@@ -478,17 +481,21 @@ out:
  * gcm_window_set_parent_xid:
  **/
 static void
-gcm_window_set_parent_xid (GtkWindow *window, guint32 xid)
+gcm_window_set_parent_xid (GtkWindow *window, guint32 _xid)
 {
 	GdkDisplay *display;
 	GdkWindow *parent_window;
 	GdkWindow *our_window;
 
 	display = gdk_display_get_default ();
-	parent_window = gdk_window_foreign_new_for_display (display, xid);
+	parent_window = gdk_window_foreign_new_for_display (display, _xid);
+	if (parent_window == NULL) {
+		g_warning ("failed to get parent window");
+		return;
+	}
 	our_window = gtk_widget_get_window (GTK_WIDGET (window));
 	if (our_window == NULL) {
-		egg_warning ("failed to get our window");
+		g_warning ("failed to get our window");
 		return;
 	}
 
@@ -517,7 +524,7 @@ gcm_viewer_add_profiles_columns (GcmViewerPrivate *viewer, GtkTreeView *treeview
 	gtk_widget_set_size_request (GTK_WIDGET (treeview), GCM_VIEWER_TREEVIEW_WIDTH, -1);
 
 	/* column for text */
-	renderer = gcm_cell_renderer_profile_new ();
+	renderer = gcm_cell_renderer_profile_text_new ();
 	g_object_set (renderer,
 		      "wrap-mode", PANGO_WRAP_WORD,
 		      "wrap-width", GCM_VIEWER_TREEVIEW_WIDTH - 62,
@@ -630,10 +637,6 @@ gcm_viewer_profiles_treeview_clicked_cb (GtkTreeSelection *selection, GcmViewerP
 	GcmProfile *profile;
 	GcmClut *clut_trc = NULL;
 	GcmClut *clut_vcgt = NULL;
-	GcmXyz *white;
-	GcmXyz *red;
-	GcmXyz *green;
-	GcmXyz *blue;
 	const gchar *profile_copyright;
 	const gchar *profile_manufacturer;
 	const gchar *profile_model ;
@@ -649,13 +652,13 @@ gcm_viewer_profiles_treeview_clicked_cb (GtkTreeSelection *selection, GcmViewerP
 	gboolean ret;
 	gboolean has_vcgt;
 	guint size = 0;
+	guint temperature;
 	guint filesize;
-	gfloat x;
 	gboolean show_section = FALSE;
 
 	/* This will only work in single or browse selection mode! */
 	if (!gtk_tree_selection_get_selected (selection, &model, &iter)) {
-		egg_debug ("no row selected");
+		g_debug ("no row selected");
 		return;
 	}
 
@@ -684,27 +687,8 @@ gcm_viewer_profiles_treeview_clicked_cb (GtkTreeSelection *selection, GcmViewerP
 		gcm_image_set_abstract_profile (GCM_IMAGE(viewer->preview_widget_output), NULL);
 	}
 
-	/* get the new details from the profile */
-	g_object_get (profile,
-		      "white", &white,
-		      "red", &red,
-		      "green", &green,
-		      "blue", &blue,
-		      NULL);
-
-	/* check we have enough data for the CIE widget */
-	x = gcm_xyz_get_x (red);
-	widget = GTK_WIDGET (gtk_builder_get_object (viewer->builder, "vbox_cie_axis"));
-	if (x > 0.001) {
-		g_object_set (viewer->cie_widget,
-			      "white", white,
-			      "red", red,
-			      "green", green,
-			      "blue", blue,
-			      NULL);
-	} else {
-		gtk_widget_hide (widget);
-	}
+	/* setup cie widget */
+	gcm_cie_widget_set_from_profile (viewer->cie_widget, profile);
 
 	/* get curve data */
 	clut_trc = gcm_profile_generate_curve (profile, 256);
@@ -781,6 +765,15 @@ gcm_viewer_profiles_treeview_clicked_cb (GtkTreeSelection *selection, GcmViewerP
 	gtk_label_set_label (GTK_LABEL (widget), temp);
 	g_free (temp);
 
+	/* set whitepoint */
+	temperature = gcm_profile_get_temperature (profile);
+	widget = GTK_WIDGET (gtk_builder_get_object (viewer->builder, "label_temp"));
+	temp = g_strdup_printf ("%i°K", temperature);
+	gtk_label_set_label (GTK_LABEL (widget), temp);
+	g_free (temp);
+	widget = GTK_WIDGET (gtk_builder_get_object (viewer->builder, "hbox_temp"));
+	gtk_widget_set_visible (widget, (temperature > 0));
+
 	/* set size */
 	widget = GTK_WIDGET (gtk_builder_get_object (viewer->builder, "hbox_size"));
 	filesize = gcm_profile_get_size (profile);
@@ -863,38 +856,35 @@ gcm_viewer_profiles_treeview_clicked_cb (GtkTreeSelection *selection, GcmViewerP
 		g_object_unref (clut_trc);
 	if (clut_vcgt != NULL)
 		g_object_unref (clut_vcgt);
-	g_object_unref (white);
-	g_object_unref (red);
-	g_object_unref (green);
-	g_object_unref (blue);
 	g_free (size_text);
 	g_free (basename);
 }
 
 /**
- * gcm_viewer_set_combo_simple_text:
+ * gcm_viewer_profile_store_added_cb:
  **/
 static void
-gcm_viewer_set_combo_simple_text (GtkWidget *combo_box)
+gcm_viewer_profile_store_added_cb (GcmProfileStore *profile_store, GcmProfile *profile, GcmViewerPrivate *viewer)
 {
-	GtkCellRenderer *renderer;
-	GtkListStore *store;
+	gcm_viewer_update_profile_list (viewer);
+}
 
-	store = gtk_list_store_new (4, G_TYPE_STRING, GCM_TYPE_PROFILE, G_TYPE_UINT, G_TYPE_STRING);
-	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (store), GCM_VIEWER_COMBO_COLUMN_SORTABLE, GTK_SORT_ASCENDING);
-	gtk_combo_box_set_model (GTK_COMBO_BOX (combo_box), GTK_TREE_MODEL (store));
-	g_object_unref (store);
+/**
+ * gcm_viewer_profile_store_removed_cb:
+ **/
+static void
+gcm_viewer_profile_store_removed_cb (GcmProfileStore *profile_store, GcmProfile *profile, GcmViewerPrivate *viewer)
+{
+	gcm_viewer_update_profile_list (viewer);
+}
 
-	renderer = gtk_cell_renderer_text_new ();
-	g_object_set (renderer,
-		      "ellipsize", PANGO_ELLIPSIZE_END,
-		      "wrap-mode", PANGO_WRAP_WORD_CHAR,
-		      "width-chars", 60,
-		      NULL);
-	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo_box), renderer, TRUE);
-	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combo_box), renderer,
-					"text", GCM_VIEWER_COMBO_COLUMN_TEXT,
-					NULL);
+/**
+ * gcm_viewer_profile_store_changed_cb:
+ **/
+static void
+gcm_viewer_profile_store_changed_cb (GcmProfileStore *profile_store, GcmViewerPrivate *viewer)
+{
+	gcm_viewer_update_profile_list (viewer);
 }
 
 /**
@@ -916,7 +906,12 @@ gcm_viewer_startup_phase1_idle_cb (GcmViewerPrivate *viewer)
 
 	/* search the disk for profiles */
 	gcm_profile_store_search (viewer->profile_store, search_flags);
-	g_signal_connect (viewer->profile_store, "changed", G_CALLBACK(gcm_viewer_profile_store_changed_cb), viewer);
+	g_signal_connect (viewer->profile_store, "changed",
+			  G_CALLBACK(gcm_viewer_profile_store_changed_cb), viewer);
+	g_signal_connect (viewer->profile_store, "added",
+			  G_CALLBACK (gcm_viewer_profile_store_added_cb), viewer);
+	g_signal_connect (viewer->profile_store, "removed",
+			  G_CALLBACK (gcm_viewer_profile_store_removed_cb), viewer);
 
 	/* update list of profiles */
 	gcm_viewer_update_profile_list (viewer);
@@ -948,16 +943,6 @@ gcm_viewer_setup_drag_and_drop (GtkWidget *widget)
 	g_free (entry.target);
 }
 
-/**
- * gcm_viewer_profile_store_changed_cb:
- **/
-static void
-gcm_viewer_profile_store_changed_cb (GcmProfileStore *profile_store, GcmViewerPrivate *viewer)
-{
-	/* clear and update the profile list */
-	gcm_viewer_update_profile_list (viewer);
-}
-
 /**
  * gcm_viewer_graph_combo_changed_cb:
  **/
@@ -1016,20 +1001,20 @@ gcm_viewer_setup_graph_combobox (GcmViewerPrivate *viewer, GtkWidget *widget)
 	gint active;
 
 	/* TRANSLATORS: combo-entry, no graph selected to be shown */
-	gtk_combo_box_append_text (GTK_COMBO_BOX(widget), _("None"));
+	gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT(widget), _("None"));
 
 	/* TRANSLATORS: combo-entry, this is a graph plot type (look it up on google...) */
-	gtk_combo_box_append_text (GTK_COMBO_BOX(widget), _("CIE 1931 xy"));
+	gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT(widget), _("CIE 1931 xy"));
 
 	/* TRANSLATORS: combo-entry, this is a graph plot type (what goes in, v.s. what goes out) */
-	gtk_combo_box_append_text (GTK_COMBO_BOX(widget), _("Transfer response curve"));
+	gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT(widget), _("Transfer response curve"));
 
 	/* TRANSLATORS: combo-entry, this is a graph plot type (what data we snd the graphics card) */
-	gtk_combo_box_append_text (GTK_COMBO_BOX(widget), _("Video card gamma table"));
+	gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT(widget), _("Video card gamma table"));
 
 	/* TRANSLATORS: combo-entry, this is a preview image of what the profile looks like */
-	gtk_combo_box_append_text (GTK_COMBO_BOX(widget), _("Image preview (from sRGB)"));
-	gtk_combo_box_append_text (GTK_COMBO_BOX(widget), _("Image preview (to sRGB)"));
+	gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT(widget), _("Image preview (from sRGB)"));
+	gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT(widget), _("Image preview (to sRGB)"));
 
 	/* get from settings */
 	active = g_settings_get_enum (viewer->settings, GCM_SETTINGS_PROFILE_GRAPH_TYPE);
@@ -1037,47 +1022,28 @@ gcm_viewer_setup_graph_combobox (GcmViewerPrivate *viewer, GtkWidget *widget)
 }
 
 /**
- * main:
+ * gcm_viewer_activate_cb:
  **/
-int
-main (int argc, char **argv)
+static void
+gcm_viewer_activate_cb (GApplication *application, GcmViewerPrivate *viewer)
+{
+	GtkWindow *window;
+	window = GTK_WINDOW (gtk_builder_get_object (viewer->builder, "dialog_viewer"));
+	gtk_window_present (window);
+}
+
+/**
+ * gcm_viewer_startup_cb:
+ **/
+static void
+gcm_viewer_startup_cb (GApplication *application, GcmViewerPrivate *viewer)
 {
-	guint retval = 0;
-	GOptionContext *context;
 	GtkWidget *main_window;
 	GtkWidget *widget;
-	guint xid = 0;
 	GError *error = NULL;
 	GtkTreeSelection *selection;
 	GdkScreen *screen;
-	GcmViewerPrivate *viewer;
-
-	const GOptionEntry options[] = {
-		{ "parent-window", 'p', 0, G_OPTION_ARG_INT, &xid,
-		  /* TRANSLATORS: we can make this modal (stay on top of) another window */
-		  _("Set the parent window to make this modal"), NULL },
-		{ NULL}
-	};
-
-	setlocale (LC_ALL, "");
-
-	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
-	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-	textdomain (GETTEXT_PACKAGE);
-
-	gtk_init (&argc, &argv);
-
-	context = g_option_context_new ("gnome-color-manager prefs program");
-	g_option_context_add_main_entries (context, options, NULL);
-	g_option_context_add_group (context, egg_debug_get_option_group ());
-	g_option_context_add_group (context, gtk_get_option_group (TRUE));
-	g_option_context_parse (context, &argc, &argv, NULL);
-	g_option_context_free (context);
-
-	viewer = g_new0 (GcmViewerPrivate, 1);
-
-	/* ensure single instance */
-	viewer->application = gtk_application_new ("org.gnome.ColorManager.Profile", &argc, &argv);
+	gint retval;
 
 	/* setup defaults */
 	viewer->settings = g_settings_new (GCM_SETTINGS_SCHEMA);
@@ -1086,7 +1052,7 @@ main (int argc, char **argv)
 	viewer->builder = gtk_builder_new ();
 	retval = gtk_builder_add_from_file (viewer->builder, GCM_DATA "/gcm-viewer.ui", &error);
 	if (retval == 0) {
-		egg_warning ("failed to load ui: %s", error->message);
+		g_warning ("failed to load ui: %s", error->message);
 		g_error_free (error);
 		goto out;
 	}
@@ -1160,7 +1126,6 @@ main (int argc, char **argv)
 
 	/* hide widgets by default */
 	widget = GTK_WIDGET (gtk_builder_get_object (viewer->builder, "combobox_graph"));
-	gcm_viewer_set_combo_simple_text (widget);
 	gcm_viewer_setup_graph_combobox (viewer, widget);
 	g_signal_connect (widget, "changed",
 			  G_CALLBACK (gcm_viewer_graph_combo_changed_cb), viewer);
@@ -1219,8 +1184,12 @@ main (int argc, char **argv)
 
 	/* set the parent window if it is specified */
 	if (xid != 0) {
-		egg_debug ("Setting xid %i", xid);
+		g_debug ("Setting xid %i", xid);
 		gcm_window_set_parent_xid (GTK_WINDOW (main_window), xid);
+
+		/* hide the preferences button */
+		widget = GTK_WIDGET (gtk_builder_get_object (viewer->builder, "button_preferences"));
+		gtk_widget_hide (widget);
 	}
 
 	/* refresh UI */
@@ -1229,10 +1198,54 @@ main (int argc, char **argv)
 
 	/* do all this after the window has been set up */
 	g_idle_add ((GSourceFunc) gcm_viewer_startup_phase1_idle_cb, viewer);
+out:
+	return;
+}
+
+/**
+ * main:
+ **/
+int
+main (int argc, char **argv)
+{
+	GOptionContext *context;
+	GcmViewerPrivate *viewer;
+	int status = 0;
+
+	const GOptionEntry options[] = {
+		{ "parent-window", 'p', 0, G_OPTION_ARG_INT, &xid,
+		  /* TRANSLATORS: we can make this modal (stay on top of) another window */
+		  _("Set the parent window to make this modal"), NULL },
+		{ NULL}
+	};
+
+	setlocale (LC_ALL, "");
+
+	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+	textdomain (GETTEXT_PACKAGE);
+
+	gtk_init (&argc, &argv);
+
+	context = g_option_context_new ("gnome-color-manager prefs program");
+	g_option_context_add_main_entries (context, options, NULL);
+	g_option_context_add_group (context, gcm_debug_get_option_group ());
+	g_option_context_add_group (context, gtk_get_option_group (TRUE));
+	g_option_context_parse (context, &argc, &argv, NULL);
+	g_option_context_free (context);
+
+	viewer = g_new0 (GcmViewerPrivate, 1);
+
+	/* ensure single instance */
+	viewer->application = gtk_application_new ("org.gnome.ColorManager.Profile", 0);
+	g_signal_connect (viewer->application, "startup",
+			  G_CALLBACK (gcm_viewer_startup_cb), viewer);
+	g_signal_connect (viewer->application, "activate",
+			  G_CALLBACK (gcm_viewer_activate_cb), viewer);
 
 	/* wait */
-	gtk_application_run (viewer->application);
-out:
+	status = g_application_run (G_APPLICATION (viewer->application), argc, argv);
+
 	g_object_unref (viewer->application);
 	if (viewer->settings != NULL)
 		g_object_unref (viewer->settings);
@@ -1241,5 +1254,5 @@ out:
 	if (viewer->profile_store != NULL)
 		g_object_unref (viewer->profile_store);
 	g_free (viewer);
-	return retval;
+	return status;
 }
